@@ -120,8 +120,8 @@ sub writeBinaryFile($$)
 # Tabularize text - basic version
 #-------------------------------------------------------------------------------
 
-sub formatTableBasic($)
- {my ($data) = @_;
+sub formatTableBasic($;$)
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   my $d = $data;
   ref($d) =~ /array/i or confess "Array reference required";
   my @D;
@@ -150,31 +150,32 @@ sub formatTableBasic($)
     push @t, $t;
    }
 
-  join "\n", @t
+  my $s = $separator//"\n";
+  join($s, @t).$s
  }
 
 #-------------------------------------------------------------------------------
 # Tabularize text depending on structure
 #-------------------------------------------------------------------------------
 
-sub formatTableAA($)                                                            # Array of Arrays
- {my ($data) = @_;
+sub formatTableAA($;$)                                                          # Array of Arrays
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   return dump($data) unless ref($data) =~ /array/i and @$data;
   my $d = [['', @{$data->[0]}]];                                                # Column headers are row 0
   push @$d, [$_, @{$data->[$_]}] for 1..$#$data;
-  formatTableBasic($d);
+  formatTableBasic($d, $separator);
  }
 
-sub formatTableHA($)                                                            # Hash of Arrays
- {my ($data) = @_;
+sub formatTableHA($;$)                                                          # Hash of Arrays
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   return dump($data) unless ref($data) =~ /hash/i and keys %$data;
   my $d;                                                                        # Column headers are row 0
   push @$d, [$_, @{$data->{$_}}] for sort keys %$data;
-  formatTableBasic($d);
+  formatTableBasic($d, $separator);
  }
 
-sub formatTableAH($)                                                            # Array of hashes
- {my ($data) = @_;
+sub formatTableAH($;$)                                                          # Array of hashes
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   return dump($data) unless ref($data) =~ /array/i and @$data;
 
   my %k; @k{keys %$_}++ for @$data;                                             # Column headers
@@ -187,11 +188,11 @@ sub formatTableAH($)
     my %h = %{$data->[$_-1]};
     $d->[-1][$k{$_}] = $h{$_} for keys %h;
    }
-  formatTableBasic($d);
+  formatTableBasic($d, $separator);
  }
 
-sub formatTableHH($)                                                            # Hash of hashes
- {my ($data) = @_;
+sub formatTableHH($;$)                                                          # Hash of hashes
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   return dump($data) unless ref($data) =~ /hash/i and keys %$data;
 
   my %k; @k{keys %$_}++ for values %$data;                                      # Column headers
@@ -204,33 +205,33 @@ sub formatTableHH($)
     my %h = %{$data->{$_}};
     $d->[-1][$k{$_}] = $h{$_} for keys %h;
    }
-  formatTableBasic($d);
+  formatTableBasic($d, $separator);
  }
 
-sub formatTableA($)                                                             # Array with mixed elements
- {my ($data) = @_;
+sub formatTableA($;$)                                                           # Array with mixed elements
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   return dump($data) unless ref($data) =~ /array/i and @$data;
 
   my $d;
   for(keys @$data)
    {push @$d, @$data > 1 ? [$_, $data->[$_]] : [$data->[$_]];                   # Skip line number if the array is degenerate
    }
-  formatTableBasic($d);
+  formatTableBasic($d, $separator);
  }
 
-sub formatTableH($)                                                             # Hash with mixed elements
- {my ($data) = @_;
+sub formatTableH($;$)                                                           # Hash with mixed elements
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   return dump($data) unless ref($data) =~ /hash/i and keys %$data;
 
   my $d;
   for(sort keys %$data)
    {push @$d, [$_, $data->{$_}];
    }
-  formatTableBasic($d);
+  formatTableBasic($d, $separator);
  }
 
-sub formatTable($)                                                              # Format various data structures
- {my ($data) = @_;
+sub formatTable($;$)                                                            # Format various data structures
+ {my ($data, $separator) = @_;                                                  # Data to be formatted, optional line separator
   my ($a, $h, $o) = (0, 0, 0);
   my $checkStructure = sub
    {for(@_)
@@ -240,29 +241,20 @@ sub formatTable($)
    };
   if    (ref($data) =~ /array/i)
    {$checkStructure->(       @$data);
-    return formatTableAA($data) if  $a and !$h and !$o;
-    return formatTableAH($data) if !$a and  $h and !$o;
-    return formatTableA ($data);
+    return formatTableAA($data, $separator) if  $a and !$h and !$o;
+    return formatTableAH($data, $separator) if !$a and  $h and !$o;
+    return formatTableA ($data, $separator);
    }
   elsif (ref($data) =~ /hash/i)
    {$checkStructure->(values %$data);
-    return formatTableHA($data) if  $a and !$h and !$o;
-    return formatTableHH($data) if !$a and  $h and !$o;
-    return formatTableH ($data);
+    return formatTableHA($data, $separator) if  $a and !$h and !$o;
+    return formatTableHH($data, $separator) if !$a and  $h and !$o;
+    return formatTableH ($data, $separator);
    }
  }
 
 #-------------------------------------------------------------------------------
-# Indent lines contained in a string by the specified amount
-#-------------------------------------------------------------------------------
-
-sub indentString($$)
- {my ($string, $indent) = @_;                                                   # The string of lines to indent, the indenting string
-  join "\n", map {$indent.$_} split "\n", $string
- }
-
-#-------------------------------------------------------------------------------
-# Load an array from lines of text in a striung
+# Load an array from lines of text in a string
 #-------------------------------------------------------------------------------
 
 sub loadArrayFromLines($)
@@ -298,15 +290,6 @@ sub loadHashArrayFromLines($)
   +{map{my @a = split /\s+/; (shift @a, [@a])} split "\n", $string}
  }
 
-#-------------------------------------------------------------------------------
-# Normalize white space
-#-------------------------------------------------------------------------------
-
-sub normalizeWhiteSpace($)
- {my ($string) = @_;                                                            # The string in which to normalize the whitespace
-  $string =~ s/\s+/ /gr =~ s/\A\s+//r =~ s/\s+\Z//r
- }
-
 #-------------------------------------------------------------------------------
 # Check the keys in a hash
 #-------------------------------------------------------------------------------
@@ -325,11 +308,51 @@ sub checkKeys($$)
     indentString(formatTable($permitted),         '  ');
  }
 
+#-------------------------------------------------------------------------------
+# Generate LVALUE methods
+#-------------------------------------------------------------------------------
+
+sub genLValueScalarMethods(@)                                                   # Scalar methods
+ {my ($package) = caller;                                                       # Package
+  for(@_)                                                                       # Name each method
+   {my $s = 'sub '.$package.'::'.$_.':lvalue {my $v = "'.$_.'"; $_[0]{"'.$_.'"} //= $v}';
+    eval $s;
+    confess "Unable to create LValue scalar method for: '$_' because\n$@" if $@;
+   }
+ }
+
+sub genLValueArrayMethods(@)                                                    # Array methods
+ {my ($package) = caller;                                                       # Package
+  for(@_)                                                                       # Name each method
+   {my $s = 'sub '.$package.'::'.$_.':lvalue {$_[0]{"'.$_.'"} //= []}';
+    eval $s;
+    confess "Unable to create LValue array method for: '$_' because\n$@" if $@;
+   }
+ }
+
+sub genLValueHashMethods(@)                                                     # Hash methods
+ {my ($package) = caller;                                                       # Package
+  for(@_)                                                                       # Name each method
+   {my $s = 'sub '.$package.'::'.$_.':lvalue {$_[0]{"'.$_.'"} //= {}}';
+    eval $s;
+    confess "Unable to create LValue array method for: '$_' because\n$@" if $@;
+   }
+ }
+
+#-------------------------------------------------------------------------------
+# Indent lines contained in a string or formatted table by the specified amount
+#-------------------------------------------------------------------------------
+
+sub indentString($$)
+ {my ($string, $indent) = @_;                                                   # The string of lines to indent, the indenting string
+  join "\n", map {$indent.$_} split "\n", (ref($string) ? $$string  : $string)
+ }
+
 #-------------------------------------------------------------------------------
 # Examples
 #-------------------------------------------------------------------------------
 
-if (0)
+if (0 and !caller)
  {say STDERR "\n","\nsay STDERR formatTable(",dump($_), ");\n# ", formatTable($_) =~ s/\n/\n# /gr for
 [[qw(. aa bb cc)], [qw(1 A B C)], [qw(2 AA BB CC)], [qw(3 AAA BBB CCC)],  [qw(4 1 22 333)]],
 [{aa=>'A', bb=>'B', cc=>'C'}, {aa=>'AA', bb=>'BB', cc=>'CC'}, {aa=>'AAA', bb=>'BBB', cc=>'CCC'}, {aa=>'1', bb=>'22', cc=>'333'}],
@@ -359,13 +382,14 @@ use vars qw(@ISA @EXPORT @EXPORT_OK %EXPORT_TAGS $VERSION);
 @ISA          = qw(Exporter);
 @EXPORT       = qw(formatTable);
 @EXPORT_OK    = qw(checkKeys dateStamp dateTimeStamp fileList fileSize
+genLValueArrayMethods genLValueHashMethods genLValueScalarMethods
 indentString
 loadArrayArrayFromLines loadArrayFromLines  loadHashArrayFromLines loadHashFromLines
-makePath normalizeWhiteSpace
+makePath
 readBinaryFile readFile timeStamp writeBinaryFile writeFile);
-%EXPORT_TAGS  = (all=>[@EXPORT_OK]);
+%EXPORT_TAGS  = (all=>[@EXPORT, @EXPORT_OK]);
 
-$VERSION = '1.004'; # Wednesday 14 December 2016
+$VERSION = '1.006'; # Thursday 15 December 2016
 
 1;
 
@@ -384,57 +408,61 @@ Data::Table::Text - Write data in tabular text format
    [1, "A", "B", "C"],
    [2, "AA", "BB", "CC"],
    [3, "AAA", "BBB", "CCC"],
-   [4, 1, 22, 333],
- ]);
+   [4, 1, 22, 333]]);
+
  #    .  aa   bb   cc
  # 1  1  A    B    C
  # 2  2  AA   BB   CC
  # 3  3  AAA  BBB  CCC
  # 4  4    1   22  333
 
+
  say STDERR formatTable([
    { aa => "A", bb => "B", cc => "C" },
    { aa => "AA", bb => "BB", cc => "CC" },
    { aa => "AAA", bb => "BBB", cc => "CCC" },
-   { aa => 1, bb => 22, cc => 333 },
- ]);
+   { aa => 1, bb => 22, cc => 333 }]);
+
  #    aa   bb   cc
  # 1  A    B    C
  # 2  AA   BB   CC
  # 3  AAA  BBB  CCC
  # 4    1   22  333
 
+
  say STDERR formatTable({
    "" => ["aa", "bb", "cc"],
    "1" => ["A", "B", "C"],
    "22" => ["AA", "BB", "CC"],
    "333" => ["AAA", "BBB", "CCC"],
-   "4444" => [1, 22, 333],
- });
+   "4444" => [1, 22, 333]});
+
  #       aa   bb   cc
  #    1  A    B    C
  #   22  AA   BB   CC
  #  333  AAA  BBB  CCC
  # 4444    1   22  333
 
+
  say STDERR formatTable({
    a => { aa => "A", bb => "B", cc => "C" },
    aa => { aa => "AA", bb => "BB", cc => "CC" },
    aaa => { aa => "AAA", bb => "BBB", cc => "CCC" },
-   aaaa => { aa => 1, bb => 22, cc => 333 },
- });
+   aaaa => { aa => 1, bb => 22, cc => 333 }});
  #       aa   bb   cc
  # a     A    B    C
  # aa    AA   BB   CC
  # aaa   AAA  BBB  CCC
  # aaaa    1   22  333
 
+
  say STDERR formatTable(["a", "bb", "ccc", 4444]);
  # 0  a
  # 1  bb
  # 2  ccc
  # 3  4444
 
+
  say STDERR formatTable({ aa => "A", bb => "B", cc => "C" });
  # aa  A
  # bb  B
@@ -471,7 +499,7 @@ modified under the same terms as Perl itself.
 =cut
 
 __DATA__
-use Test::More tests => 24;
+use Test::More tests => 29;
 if (1)                                                                          # Unicode
  {use utf8;
   my $z = "𝝰𝝱𝝲";
@@ -505,42 +533,34 @@ if (1)
  }
 if (1)                                                                          # Format table and AA
  {my $d = [[qw(. aa bb cc)], [qw(1 A B C)], [qw(2 AA BB CC)], [qw(3 AAA BBB CCC)],  [qw(4 1 22 333)]];
-  ok formatTableBasic($d) =~ s/\n/ /gr eq '.  aa   bb   cc    1  A    B    C     2  AA   BB   CC    3  AAA  BBB  CCC   4    1   22  333  ';
-  ok formatTable     ($d) =~ s/\n/ /gr eq '   .  aa   bb   cc    1  1  A    B    C     2  2  AA   BB   CC    3  3  AAA  BBB  CCC   4  4    1   22  333  ';
+  ok formatTableBasic($d, '|') eq '.  aa   bb   cc   |1  A    B    C    |2  AA   BB   CC   |3  AAA  BBB  CCC  |4    1   22  333  |';
+  ok formatTable($d, '|') eq '   .  aa   bb   cc   |1  1  A    B    C    |2  2  AA   BB   CC   |3  3  AAA  BBB  CCC  |4  4    1   22  333  |';
  }
 if (1)                                                                          # AH
  {my $d = [{aa=>'A', bb=>'B', cc=>'C'}, {aa=>'AA', bb=>'BB', cc=>'CC'}, {aa=>'AAA', bb=>'BBB', cc=>'CCC'}, {aa=>'1', bb=>'22', cc=>'333'}];
-  ok formatTable     ($d) =~ s/\n/ /gr eq '   aa   bb   cc    1  A    B    C     2  AA   BB   CC    3  AAA  BBB  CCC   4    1   22  333  ';
+  ok formatTable($d, '|') eq '   aa   bb   cc   |1  A    B    C    |2  AA   BB   CC   |3  AAA  BBB  CCC  |4    1   22  333  |';
  }
 if (1)                                                                          # HA
  {my $d = {''=>[qw(aa bb cc)], 1=>[qw(A B C)], 22=>[qw(AA BB CC)], 333=>[qw(AAA BBB CCC)],  4444=>[qw(1 22 333)]};
-  ok formatTable     ($d) =~ s/\n/ /gr eq '      aa   bb   cc       1  A    B    C       22  AA   BB   CC     333  AAA  BBB  CCC   4444    1   22  333  ';
+  ok formatTable($d, '|') eq '      aa   bb   cc   |   1  A    B    C    |  22  AA   BB   CC   | 333  AAA  BBB  CCC  |4444    1   22  333  |';
  }
 if (1)                                                                          # HH
  {my $d = {a=>{aa=>'A', bb=>'B', cc=>'C'}, aa=>{aa=>'AA', bb=>'BB', cc=>'CC'}, aaa=>{aa=>'AAA', bb=>'BBB', cc=>'CCC'}, aaaa=>{aa=>'1', bb=>'22', cc=>'333'}};
-  ok formatTable     ($d) =~ s/\n/ /gr eq '      aa   bb   cc    a     A    B    C     aa    AA   BB   CC    aaa   AAA  BBB  CCC   aaaa    1   22  333  ';
+  ok formatTable($d, '|') eq '      aa   bb   cc   |a     A    B    C    |aa    AA   BB   CC   |aaa   AAA  BBB  CCC  |aaaa    1   22  333  |';
  }
 if (1)                                                                          # A
  {my $d = [qw(a bb ccc 4444)];
-  ok formatTable     ($d) =~ s/\n/ /gr eq '0  a      1  bb     2  ccc    3  4444  ';
+  ok formatTable($d, '|') eq '0  a     |1  bb    |2  ccc   |3  4444  |';
  }
 if (1)                                                                          # H
  {my $d = {aa=>'A', bb=>'B', cc=>'C'};
-  ok formatTable     ($d) =~ s/\n/ /gr eq 'aa  A   bb  B   cc  C  ';
+  ok formatTable($d, '|') eq 'aa  A  |bb  B  |cc  C  |';
  }
 
-if (1)                                                                          # H
- {my $s = normalizeWhiteSpace
-    checkKeys({qw(go 1 Went 2 gone 3)},
-              {qw(go present went past gone presentPast)});
-  my $S = normalizeWhiteSpace <<END;
-Invalid options chosen:
-  Went
-Permitted options are:
-  go    present
-  gone  presentPast
-  went  past
-END
+if (1)                                                                          # CheckKeys, nsw
+ {my $s = checkKeys({qw(go 1 Went 2 gone 3)},
+                    {qw(go present went past gone presentPast)}) =~ s/\n/|/gr;
+  my $S = 'Invalid options chosen:|  Went  ||Permitted options are:|  go    present      |  gone  presentPast  |  went  past         ';
   ok $s eq $S;
  }
 
@@ -575,3 +595,34 @@ b AA BB CC
 END
   ok dump($s) eq '{ a => ["A", "B", "C"], b => ["AA", "BB", "CC"] }';
  }
+
+if (1)                                                                          # SM
+ {package Scalars;
+  my $a = bless{};
+  Data::Table::Text::genLValueScalarMethods(qw(aa bb cc));
+  Test::More::ok $a->aa eq 'aa';
+  $a->aa = 'bb';
+  Test::More::ok $a->aa eq 'bb';
+ }
+
+if (1)                                                                          # AM
+ {package Arrays;
+  my $a = bless{};
+  Data::Table::Text::genLValueArrayMethods(qw(aa bb cc));
+  $a->aa->[1] = 'aa';
+  Test::More::ok $a->aa->[1] eq 'aa';
+ }
+
+if (1)                                                                          # AM
+ {package Hashes;
+  my $a = bless{};
+  Data::Table::Text::genLValueHashMethods(qw(aa bb cc));
+  $a->aa->{a} = 'aa';
+  Test::More::ok $a->aa->{a} eq 'aa';
+ }
+
+if (1)                                                                          # indentString
+ {my $d = [[qw(. aa bb cc)], [qw(1 A B C)], [qw(2 AA BB CC)], [qw(3 AAA BBB CCC)],  [qw(4 1 22 333)]];
+  my $s = indentString(formatTable($d), '  ') =~ s/\n/|/gr;
+  ok $s eq '     .  aa   bb   cc   |  1  1  A    B    C    |  2  2  AA   BB   CC   |  3  3  AAA  BBB  CCC  |  4  4    1   22  333  ';
+ }
